// Arduino code for dynamic playseat 2DOF
//     Created 24 May 2011 by Jim Lindblom   SparkFun Electronics https://www.sparkfun.com/products/10182 "Example Code"
 //    Created 24 Apr 2012 by Jean David SEDRUE Version betatest26 - 24042012 http://www.gamoover.net/Forums/index.php?topic=25907
 //    Updated 20 May 2013 by RacingMat in english http://www.x-sim.de/forum/posting.php?mode=edit&f=37&t=943&p=8481  in french : http://www.gamoover.net/Forums/index.php?topic=27617
  //   Updated 30 April 2014 by RacingMat (bug for value below 16 corrected)
  //   Updated 16  Décembre 2015 by Tian30 (bug sur -87% à-100% envoyé de simtools : un autre bug sur les valeurs inférieures à 10 en hexa = 16 en décimales, traitées correctement sur le premier de 3 datas, mais qui engendraient forcément une erreur sur le lot suivant. 
  // Le paragraphe "void readSerialData()" a été modifié, ainsi que le traitement "int NormalizeData(byte x[3])"
/*
____________carte1__________________
   NANO       Monster  
  violet            blanc
A4--->MOT2R     A5--->MOT1L      
*D8/11 (jaune     )inb2 P9  MOT2
  D4/7(  gris      ) inb1 P8  MOT1
  D7/10(  noir     ) ina1 P7  MOT1         
*D6/9  (  bleu    )pwm2 P6  MOT2 
  D5/8 (  vert      )pwm1 P5  MOT1
*D2/5  ( blanc    )ina2 P4  MOT2

*/
#define BRAKEVCC 0
#define RV  1 //méfiez-vous  en fonction du câblage de matériel
#define FW  2 //méfiez-vous en fonction du câblage de matériel
#define STOP 0
#define BRAKEGND 3


////////////////////////////////////////////////////////////////////////////////
#define pwmMax 255 // ou moins , si vous voulez réduire la vitesse du moteur maximale

// définir la plage de rotation du potentiomètre 
const int potMini=5;//5
const int potMaxi=1000;


////////////////////////////////////////////////////////////////////////////////
#define motLeft 0
#define motRight 1
#define potL A5
#define potR A3


////////////////////////////////////////////////////////////////////////////////
//  DECLARATIONS
////////////////////////////////////////////////////////////////////////////////
/*  VNH2SP30 pin definitions*/
int inApin[2] = {
  7, 11};  // INA: sens horaire
int inBpin[2] = {
  4, 12}; // INB: sens antihoraire
int pwmpin[2] = {
  5, 3}; // PWM input

/* init position value*/
int DataValueL=512; //middle position 0-1024
int DataValueR=512; //middle position 0-1024


////////////////////////////////////////////////////////////////////////////////
// INITIALIZATION
////////////////////////////////////////////////////////////////////////////////
void setup()
{
  // serial initialization
  Serial.begin(115200);


  // initialization of Arduino's pins
 


  for (int i=0; i<2; i++)
  {
    pinMode(inApin[i], OUTPUT);
    pinMode(inBpin[i], OUTPUT);
    pinMode(pwmpin[i], OUTPUT);
  }
  // Initialize braked for motor
  for (int i=0; i<2; i++)
  {
    digitalWrite(inApin[i], LOW);
    digitalWrite(inBpin[i], LOW);
  }
}
////////////////////////////////////////////////////////////////////////////////
///////////////////////////////// Main Loop ////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
void loop()
{
  int sensorL,sensorR;


  readSerialData();   // DataValueR & L contiennent le dernier ordre reçu (si il n'y a pas plus récent reçu , le dernier est maintenu )
  // L'ordre précédent sera toujours utilisé par le MotorMotion régulation PID Fonction


  sensorR = analogRead(potR);  // range 0-1024
  sensorL = analogRead(potL);  // range 0-1024

   
  motorMotion(motRight,sensorR,DataValueR);
  
  motorMotion(motLeft,sensorL,DataValueL);
}
////////////////////////////////////////////////////////////////////////////////
// Procedure: wait for complete trame
////////////////////////////////////////////////////////////////////////////////
void readSerialData()
{
  byte Data[1]={
    '0'         };// on va lire une 1ère donnée
 if (Serial.available()>0){    
  //analyser la mémoire tampon : test si l'octet est  "L" ou "R"
       Data[0]=Serial.read();
                     if (Data[0]=='L'){
                            byte Data[2]={ // on va lire deux données
                              '0','0'         };
                              if (Serial.available()>1){
                           Data[0]=Serial.read();
                           Data[1]=Serial.read(); 
                               if ((Data[1]=='R') || (Data[1]=='L')){// pour le cas où la position est sur un seul caractère, de 0 à F
                               Data[1]=Data[0];  //move MSB to LSB
                               Data[0]='0'; 
                               }
                             DataValueL=NormalizeData(Data);
                             } 
                              }
                      if (Data[0]=='R'){
                            byte Data[2]={// on va lire deux données
                              '0','0'         };
                              if (Serial.available()>1){
                           Data[0]=Serial.read();
                           Data[1]=Serial.read(); 
                            if ((Data[1]=='R') || (Data[1]=='L')){// pour le cas où la position est sur un seul caractère, de 0 à F
                               Data[1]=Data[0];  //move MSB to LSB
                               Data[0]='0'; 
                               }

                             DataValueR=NormalizeData(Data);
                             } 
                              } 
                       
                        DataValueL=DataValueL;// recherche valeur précédente si donnée 1 n'est ni "L" ni "R"
                        DataValueR=DataValueR;//
                       
                       }
                                
 
 
 
     if (Serial.available()>16) Serial.flush();
  }
  

////////////////////////////////////////////////////////
void motorMotion(int numMot,int actualPos,int targetPos)
////////////////////////////////////////////////////////
{
  int Tol=10; //20 aucun ordre de se déplacer sera envoyé au moteur si la cible est proche de la position réelle
  // Cela empêche déplacements courts jittering
  // pourrait être un paramètre lu à partir d'un pot sur une broche analogique
  // plus haute est la valeur , plus le simulateur est calme  ( moins de déplacements )

  int gap;
  int pwm;
  int brakingDistance=10;


  // problème de sécurité : TargetPos doit être dans la gamme mécanique autorisé
  targetPos=constrain(targetPos,potMini+brakingDistance,potMaxi-brakingDistance);


  gap=abs(targetPos-actualPos);


  if (gap<= Tol) {
    motorOff(numMot); //trop près à se déplacer    
  }
  else {
    // PID : vitesse calculée fonction de la distance
    pwm=20; 
    if (gap>20)  pwm=40;
    if (gap>30)  pwm=80;
    if (gap>40)  pwm=120;
    if (gap>50)  pwm=255;
    pwm=map(pwm, 0, 255, 0, pwmMax);  //ajuster la valeur en fonction de pwmMax à des fins de débogage mécanique !

    
    // Si le moteur est à l'extérieur de la gamme , envoyer moteur arrière à la limite!
    // Aller de l'avant (jusqu'à )
    if ((actualPos<potMini) || (actualPos<targetPos)) motorGo(numMot, FW, pwm);
    // allez inverser ( vers le bas)  
    if ((actualPos>potMaxi) || (actualPos>targetPos)) motorGo(numMot, RV, pwm);


  }
}






////////////////////////////////////////////////////////////////////////////////
void motorOff(int motor){ //Brake Ground : free wheel actually
  ////////////////////////////////////////////////////////////////////////////////
  digitalWrite(inApin[motor], LOW);
  digitalWrite(inBpin[motor], LOW);
  analogWrite(pwmpin[motor], 0);
}
////////////////////////////////////////////////////////////////////////////////
void motorOffBraked(int motor){ // "brake VCC" : short-circuit inducing electromagnetic brake
  ////////////////////////////////////////////////////////////////////////////////
  digitalWrite(inApin[motor], HIGH);
  digitalWrite(inBpin[motor], HIGH);
  analogWrite(pwmpin[motor], 0);
}


////////////////////////////////////////////////////////////////////////////////
void motorGo(uint8_t motor, uint8_t direct, uint8_t pwm)
////////////////////////////////////////////////////////////////////////////////
{
  if (motor <= 1)
  {
    if (direct <=4)
    {
      // Set inA[motor]
      if (direct <=1)
        digitalWrite(inApin[motor], HIGH);
      else
        digitalWrite(inApin[motor], LOW);


      // Set inB[motor]
      if ((direct==0)||(direct==2))
        digitalWrite(inBpin[motor], HIGH);
      else
        digitalWrite(inBpin[motor], LOW);


      analogWrite(pwmpin[motor], pwm);


    }
  }
}



////////////////////////////////////////////////////////////////////////////////
// Function: convert Hex to Dec
////////////////////////////////////////////////////////////////////////////////
int NormalizeData(byte x[3])
////////////////////////////////////////////////////////////////////////////////
{
  int result;


  if ((x[1]==13) )  
  {
    x[1]=x[0];  //move MSB to LSB
    x[0]='0';     //clear MSB
  }
  for (int i=0; i<2; i++)
   {
    if (x[i]>47 && x[i]<58 ){//for x0 to x9
      x[i]=x[i]-48;
    }                       
      if (x[i]>64 && x[i]<71 ){//for xA to xF
      x[i]=(x[i]-65)+10;       
    }
  }
  // cartographier la gamme de XSIM ( 0 < - 255 ) à la gamme mécanique autorisé ( potMini < - potMaxi )
  result=map((x[0]*16+x[1]),0,255,potMini,potMaxi);
  return result;
}
